@@ -10,6 +10,11 @@ import psutil
 from pint import _DEFAULT_REGISTRY as ureg
 
 from .base import Sensor, JSONSensor, version_info_type
+from .exceptions import (
+    PersistentSensorFailureError,
+    IntermittentSensorFailureError,
+    DataCollectionError,
+)
 
 
 class PythonVersion(JSONSensor[version_info_type]):
@@ -38,7 +43,6 @@ class IPAddresses(JSONSensor[t.Iterable[t.Tuple[str, str]]]):
     def value(self) -> t.List[t.Tuple[str, str]]:
         hostname = socket.gethostname()
         addresses = socket.getaddrinfo(hostname, None)
-
         address_info: t.List[t.Tuple[str, str]] = []
         for address in addresses:
             family, ip = (address[0].name, address[4][0])
@@ -87,110 +91,99 @@ class RAMAvailable(JSONSensor[int]):
         return "{:.1f} {}".format(scaled_value, cls.UNITS[magnitude])
 
 
-class ACStatus(JSONSensor[t.Optional[bool]]):
+class ACStatus(JSONSensor[bool]):
     name = "ACStatus"
     title = "AC Connected"
 
-    def value(self) -> t.Optional[bool]:
+    def value(self) -> bool:
         battery = psutil.sensors_battery()
         if battery is not None:
             value = battery.power_plugged
             if value is None:
-                return None
+                raise IntermittentSensorFailureError("Can't find AC status")
             else:
                 return bool(value)
         else:
-            return None
+            raise PersistentSensorFailureError("No charging circuit installed")
 
     @classmethod
-    def format(cls, value: t.Optional[bool]) -> str:
-        if value is None:
-            return "Unknown"
-        elif value:
+    def format(cls, value: bool) -> str:
+        if value:
             return "Connected"
         else:
             return "Not connected"
 
 
-class Temperature(Sensor[t.Optional[t.Any]]):
-    name = "Temperature"
-    title = "Ambient Temperature"
-
+class DHTSensor:
     def __init__(self) -> None:
         self.board = os.environ.get("APD_SENSORS_TEMPERATURE_BOARD", "DHT22")
         self.pin = os.environ.get("APD_SENSORS_TEMPERATURE_PIN", "D20")
 
-    def value(self) -> t.Optional[t.Any]:
+    @property
+    def sensor(self) -> t.Any:
         try:
             import adafruit_dht
             import board
 
+            # Force using legacy interface
+            adafruit_dht._USE_PULSEIO = False
+
             sensor_type = getattr(adafruit_dht, self.board)
             pin = getattr(board, self.pin)
-        except (ImportError, NotImplementedError, AttributeError):
+            return sensor_type(pin)
+        except (ImportError, NotImplementedError, AttributeError) as err:
             # No DHT library results in an ImportError.
             # Running on an unknown platform results in a
             # NotImplementedError when getting the pin
-            return None
+            raise PersistentSensorFailureError(
+                "Unable to initialise sensor interface"
+            ) from err
+
+
+class Temperature(Sensor[t.Any], DHTSensor):
+    name = "Temperature"
+    title = "Ambient Temperature"
+
+    def value(self) -> t.Any:
         try:
-            return ureg.Quantity(sensor_type(pin).temperature, ureg.celsius)
-        except RuntimeError:
-            return None
+            return ureg.Quantity(self.sensor.temperature, ureg.celsius)
+        except DataCollectionError:
+            # This is one of our own exceptions, we don't need to re-wrap it
+            raise
+        except (RuntimeError, AttributeError) as err:
+            raise IntermittentSensorFailureError(
+                "Couldn't determine temperature"
+            ) from err
 
     @classmethod
-    def format(cls, value: t.Optional[t.Any]) -> str:
-        if value is None:
-            return "Unknown"
-        else:
-            return "{:.3~P} ({:.3~P})".format(value, value.to(ureg.fahrenheit))
+    def format(cls, value: t.Any) -> str:
+        return "{:.3~P} ({:.3~P})".format(value, value.to(ureg.fahrenheit))
 
     @classmethod
-    def to_json_compatible(cls, value: t.Optional[t.Any]) -> t.Any:
-        if value is not None:
-            return {"magnitude": value.magnitude, "unit": str(value.units)}
-        else:
-            return None
+    def to_json_compatible(cls, value: t.Any) -> t.Any:
+        return {"magnitude": value.magnitude, "unit": str(value.units)}
 
     @classmethod
-    def from_json_compatible(cls, json_version: t.Any) -> t.Optional[t.Any]:
-        if json_version:
-            return ureg.Quantity(json_version["magnitude"], ureg[json_version["unit"]])
-        else:
-            return None
+    def from_json_compatible(cls, json_version: t.Any) -> t.Any:
+        return ureg.Quantity(json_version["magnitude"], ureg[json_version["unit"]])
 
     def __str__(self) -> str:
         return self.format(self.value())
 
 
-class RelativeHumidity(JSONSensor[t.Optional[float]]):
+class RelativeHumidity(JSONSensor[float], DHTSensor):
     name = "RelativeHumidity"
     title = "Relative Humidity"
 
-    def __init__(self) -> None:
-        self.board = os.environ.get("APD_SENSORS_TEMPERATURE_BOARD", "DHT22")
-        self.pin = os.environ.get("APD_SENSORS_TEMPERATURE_PIN", "D20")
-
-    def value(self) -> t.Optional[float]:
+    def value(self) -> float:
         try:
-            import adafruit_dht
-            import board
-
-            sensor_type = getattr(adafruit_dht, self.board)
-            pin = getattr(board, self.pin)
-        except (ImportError, NotImplementedError, AttributeError):
-            # No DHT library results in an ImportError.
-            # Running on an unknown platform results in a
-            # NotImplementedError when getting the pin
-            return None
-
-        try:
-            return float(sensor_type(pin).humidity)
-        except RuntimeError:
-            return None
+            return float(self.sensor.humidity)
+        except DataCollectionError:
+            # This is one of our own exceptions, we don't need to re-wrap it
+            raise
+        except (TypeError, AttributeError) as err:
+            raise IntermittentSensorFailureError("Couldn't determine humidity") from err
 
     @classmethod
-    def format(cls, value: t.Optional[float]) -> str:
-        if value is None:
-            return "Unknown"
-        else:
-            return "{:.1%}".format(value)
+    def format(cls, value: float) -> str:
+        return "{:.1%}".format(value)
