@@ -12,16 +12,16 @@ REQUIRED_CONFIG_KEYS = {"APD_SENSORS_API_KEY"}
 
 
 def require_api_key(
-    func: t.Callable[[], ViewFuncReturn]
-) -> t.Callable[[], t.Union[ViewFuncReturn, ErrorReturn]]:
+    func: t.Callable[..., ViewFuncReturn]
+) -> t.Callable[..., t.Union[ViewFuncReturn, ErrorReturn]]:
     @functools.wraps(func)
-    def wrapped() -> t.Union[ViewFuncReturn, ErrorReturn]:
+    def wrapped(*args, **kwargs) -> t.Union[ViewFuncReturn, ErrorReturn]:
         api_key = flask.current_app.config["APD_SENSORS_API_KEY"]
         headers = flask.request.headers
         supplied_key = headers.get("X-API-Key", "")
         if not compare_digest(api_key, supplied_key):
             return {"error": "Supply API key in X-API-Key header"}, 403
-        return func()
+        return func(*args, **kwargs)
 
     return wrapped
 
@@ -39,5 +39,9 @@ def set_up_config(
     missing_keys = REQUIRED_CONFIG_KEYS - environ.keys()
     if missing_keys:
         raise ValueError("Missing config variables: {}".format(", ".join(missing_keys)))
+    data_file = os.path.join(os.getcwd(), "sensor_data.sqlite")
+    environ["SQLALCHEMY_DATABASE_URI"] = environ.get(
+        "APD_SENSORS_DB_URI", f"sqlite:///{data_file}"
+    )
     to_configure.config.from_mapping(environ)
     return to_configure
