@@ -3,7 +3,8 @@ import typing as t
 
 import flask
 
-from apd.sensors.cli import get_sensors
+from apd.sensors import cli
+from apd.sensors.exceptions import DataCollectionError
 from .base import require_api_key
 
 version = flask.Blueprint(__name__, __name__)
@@ -14,13 +15,16 @@ version = flask.Blueprint(__name__, __name__)
 def sensor_values() -> t.Tuple[t.Dict[str, t.Any], int, t.Dict[str, str]]:
     headers = {"Content-Security-Policy": "default-src 'none'"}
     data = {}
-    for sensor in get_sensors():
-        value = sensor.value()
+    for sensor in cli.get_sensors():
+        try:
+            value = sensor.value()
+        except DataCollectionError:
+            value = None
         try:
             json.dumps(value)
         except TypeError:
             # This value isn't JSON serializable, skip it
             continue
         else:
-            data[sensor.title] = sensor.value()
+            data[sensor.title] = value
     return data, 200, headers
